@@ -43,9 +43,9 @@ from electrum.paymentrequest import InvoiceStore
 from electrum.contacts import Contacts
 from electrum.synchronizer import Synchronizer
 from electrum.verifier import SPV
-from electrum.util import DebugMem
+from electrum.util import DebugMem, UserCancelled
 from electrum.wallet import Abstract_Wallet
-from installwizard import InstallWizard
+from installwizard import InstallWizard, GoBack
 
 
 try:
@@ -159,14 +159,17 @@ class ElectrumGui:
                 w.bring_to_top()
                 break
         else:
-            wallet = self.daemon.load_wallet(path)
-            if not wallet:
-                wizard = InstallWizard(self.config, self.app, self.plugins, self.daemon.network, path)
+            try:
+                wallet = self.daemon.load_wallet(path)
+            except BaseException as e:
+                QMessageBox.information(None, _('Error'), str(e), _('OK'))
+                return
+            if wallet is None:
+                wizard = InstallWizard(self.config, self.app, self.plugins, path)
                 wallet = wizard.run_and_get_wallet()
                 if not wallet:
                     return
-                #if wallet.get_action():
-                #    return
+                wallet.start_threads(self.daemon.network)
                 self.daemon.add_wallet(wallet)
             w = self.create_window_for_wallet(wallet)
         if uri:
@@ -181,23 +184,35 @@ class ElectrumGui:
             self.config.save_last_wallet(window.wallet)
         run_hook('on_close_window', window)
 
+    def init_network(self):
+        # Show network dialog if config does not exist
+        if self.daemon.network:
+            if self.config.get('auto_connect') is None:
+                wizard = InstallWizard(self.config, self.app, self.plugins, None)
+                wizard.init_network(self.daemon.network)
+                wizard.terminate()
+
     def main(self):
+        try:
+            self.init_network()
+        except UserCancelled:
+            return
+        except GoBack:
+            return
+        except:
+            traceback.print_exc(file=sys.stdout)
+            return
         self.timer.start()
         self.config.open_last_wallet()
         path = self.config.get_wallet_path()
         if not self.start_new_window(path, self.config.get('url')):
             return
-
         signal.signal(signal.SIGINT, lambda *args: self.app.quit())
-
         # main loop
         self.app.exec_()
-
         # Shut down the timer cleanly
         self.timer.stop()
-
         # clipboard persistence. see http://www.mail-archive.com/pyqt@riverbankcomputing.com/msg17328.html
         event = QtCore.QEvent(QtCore.QEvent.Clipboard)
         self.app.sendEvent(self.app.clipboard(), event)
-
         self.tray.hide()
